import React, { useContext, useEffect, useCallback, useRef, useState } from 'react';
import { Linking } from 'react-native';
import socketio from 'socket.io-client';
import { SocketContextProvider, SocketContext } from './Context';
import { From, MessageTypes, CarouselDestinationTypes } from '../../types/Message';
import { useAsyncStorage } from '../AsyncStorage';
import { useChatConfigurations } from '../ChatConfigurations';
import { getHourAndMinutes } from '../../utils/getHourAndMinutes';
import { sendNotification } from '../../utils/sendLocalNotification';
import { SOCKET_URL } from '../../constants';
import api from '../../services/api';
import { useDebounce } from '../../hooks/useDebounce';
const botChannel = 'webchat';
const TWO_SECONDS = 2000;
function SocketContextComponent(_ref) {
  let {
    children,
    botId,
    params
  } = _ref;
  const clientRef = useRef(null);
  const isFirstRender = useRef(true);
  const [waitingToReconnect, setWaitingToReconnect] = useState(false);
  const {
    getItemsAsync,
    clearAsync,
    saveDataAsync
  } = useAsyncStorage();
  const {
    fetchBotAndUpdateConfigs
  } = useChatConfigurations();
  const [storageState, setStorageState] = useState(null);
  const debouncedMessages = useDebounce(storageState === null || storageState === void 0 ? void 0 : storageState.messages, TWO_SECONDS);
  const addMessage = newMessage => {
    setStorageState(prevState => {
      var _prevState$messages, _prevState$messages$f;
      const newMessages = [...((prevState === null || prevState === void 0 ? void 0 : (_prevState$messages = prevState.messages) === null || _prevState$messages === void 0 ? void 0 : (_prevState$messages$f = _prevState$messages.filter) === null || _prevState$messages$f === void 0 ? void 0 : _prevState$messages$f.call(_prevState$messages, m => m.type !== MessageTypes.TYPING)) ?? []), {
        ...newMessage,
        hour: getHourAndMinutes()
      }];
      return {
        messages: newMessages,
        sessionId: prevState === null || prevState === void 0 ? void 0 : prevState.sessionId
      };
    });
  };
  const handleSubmitMessage = useCallback(async _ref2 => {
    let {
      message,
      type,
      ext,
      localFileUri,
      document
    } = _ref2;
    const isMedia = type !== MessageTypes.TEXT;
    try {
      var _clientRef$current;
      addMessage({
        from: From.USER,
        message: isMedia ? localFileUri : message,
        type,
        document
      });
      (_clientRef$current = clientRef.current) === null || _clientRef$current === void 0 ? void 0 : _clientRef$current.emit('message', {
        botId,
        message,
        isMedia,
        ext,
        isPreview: false,
        id: storageState === null || storageState === void 0 ? void 0 : storageState.sessionId,
        botChannel
      });
    } catch (error) {
      console.log('Error sending message', error);
    }
  }, [storageState === null || storageState === void 0 ? void 0 : storageState.sessionId]);
  const startConversation = client => {
    client.emit('subscribe', {
      botId,
      isPreview: false,
      botChannel
    });
  };
  const restartConversation = useCallback(() => {
    console.log('RESTARTING');
    clearAsync().then(() => {
      console.log('CLEANED');
      console.log('client', clientRef.current);
      if (clientRef.current) {
        console.log('STARTING');
        startConversation(clientRef.current);
      }
    });
  }, [clientRef.current]);
  useEffect(() => {
    (async () => {
      if (debouncedMessages !== null && debouncedMessages !== void 0 && debouncedMessages.length && storageState !== null && storageState !== void 0 && storageState.sessionId) {
        await saveDataAsync(storageState);
      }
    })();
  }, [debouncedMessages]);
  useEffect(() => {
    getItemsAsync().then(stored => {
      setStorageState(stored);
    }).catch(console.log);
  }, []);
  useEffect(() => {
    (async () => {
      await fetchBotAndUpdateConfigs(botId);
    })();
  }, []);
  useEffect(() => {
    if (isFirstRender.current) {
      isFirstRender.current = false;
      return;
    }
    if (waitingToReconnect) return;
    const client = socketio(SOCKET_URL);
    if (!(clientRef !== null && clientRef !== void 0 && clientRef.current)) {
      clientRef.current = client;
      client.on('connect', () => {
        startConversation(client);
      });
      client.on('subscribe-response', async data => {
        if (!(storageState !== null && storageState !== void 0 && storageState.sessionId) || storageState.sessionId !== data.id) {
          const newData = {
            sessionId: data.id,
            messages: (storageState === null || storageState === void 0 ? void 0 : storageState.messages) || []
          };
          setStorageState(newData);
          await saveDataAsync(newData);
        }
        client.emit('start', {
          botId,
          id: data.id,
          isPreview: false,
          botChannel,
          parameters: JSON.stringify(params)
        });
      });
      client.on('reconnect_attempt', () => {
        client.io.opts.transports = ['polling', 'websocket'];
      });
      client.on('disconnect', () => {
        if (!clientRef.current) return;
        if (waitingToReconnect) return;
        setWaitingToReconnect(true);
        setTimeout(() => setWaitingToReconnect(false), 500);
      });
      client.on('message', data => {
        const incomingMessage = {
          ...data,
          from: From.BOT
        };
        addMessage(incomingMessage);
        if (incomingMessage.type !== MessageTypes.TYPING) {
          sendNotification({
            body: incomingMessage.message
          });
        }
      });
    }
  }, []);
  const handleOpenLink = async url => {
    await Linking.openURL(url);
  };
  const handleCarouselButtonClick = async _ref3 => {
    let {
      clickedButton,
      clickedCard
    } = _ref3;
    const {
      destination
    } = clickedButton;
    if ((destination === null || destination === void 0 ? void 0 : destination.type) === CarouselDestinationTypes.URL) {
      handleOpenLink(destination === null || destination === void 0 ? void 0 : destination.value);
    }
    if ((destination === null || destination === void 0 ? void 0 : destination.type) === CarouselDestinationTypes.PHONE) {
      var _destination$value;
      const treated = destination === null || destination === void 0 ? void 0 : (_destination$value = destination.value) === null || _destination$value === void 0 ? void 0 : _destination$value.replace(/[^A-Z0-9]/gi, '');
      handleOpenLink(`tel:+55${treated}`);
    }
    await api.post('/webchat/action', {
      botId: botId,
      action: {
        type: MessageTypes.CAROUSEL,
        data: {
          clickedButton,
          clickedCard
        }
      },
      sessionId: storageState === null || storageState === void 0 ? void 0 : storageState.sessionId
    });
  };
  return /*#__PURE__*/React.createElement(SocketContextProvider, {
    value: {
      handleCarouselButtonClick,
      handleSubmitMessage,
      messages: (storageState === null || storageState === void 0 ? void 0 : storageState.messages) || [],
      restartConversation
    }
  }, children);
}
export default SocketContextComponent;
export function useSocketContext() {
  return useContext(SocketContext);
}
//# sourceMappingURL=Component.js.map